@@ -25,7 +25,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from __future__ import unicode_literals
 
-__version__ = "0.3.52"
+__version__ = "0.3.54"
 __author__ = "nagev"
 __license__ = "GPLv3"
 
@@ -97,7 +97,7 @@ def fetch_decode(url):
 
 
 def new(url, basic=True, gdata=False, signature=True, size=False,
-        callback=None):
+        callback=lambda x: None):
     """ Return a new pafy instance given a url or video id.
 
     Optional arguments:
@@ -518,7 +518,9 @@ class Stream(object):
 
     def __init__(self, sm, parent):
         """ Set initial values. """
-        safeint = lambda x: int(x) if x.isdigit() else x
+        def safeint(x):
+            """ Return type int if x is a digit. """
+            return int(x) if x.isdigit() else x
 
         self._itag = sm['itag']
         self._threed = 'stereo3d' in sm and sm['stereo3d'] == '1'
@@ -531,7 +533,7 @@ class Stream(object):
         self._title = parent.title
         self.encrypted = 's' in sm
         self._parent = parent
-        self._filename = self.title + "." + self.extension
+        self._filename = self.generate_filename()
         self._fsize = None
         self._bitrate = self._rawbitrate = None
         self._mediatype = g.itags[self.itag][2]
@@ -547,6 +549,21 @@ class Stream(object):
             self._resolution = "0x0"
             self._rawbitrate = int(sm["bitrate"])
 
+    def generate_filename(self, meta=False):
+        """ Generate filename. """
+        ok = re.compile(r'[^/]')
+
+        if os.name == "nt":
+            ok = re.compile(r'[^\\/:*?"<>|]')
+
+        filename = "".join(x if ok.match(x) else "_" for x in self._title)
+
+        if meta:
+            filename += "-%s-%s" % (self._parent.videoid, self._itag)
+
+        filename += "." + self._extension
+        return filename
+
     @property
     def rawbitrate(self):
         """ Return raw bitrate value. """
@@ -684,18 +701,28 @@ class Stream(object):
 
         return self._fsize
 
-    def download(self, filepath="", quiet=False, callback=None):
-        """ Download.  Use quiet=True to supress output. Return filename. """
+    def download(self, filepath="", quiet=False, callback=lambda *x: None,
+                 meta=False):
+        """ Download.  Use quiet=True to supress output. Return filename.
+
+        Use meta=True to append video id and itag to generated filename
+
+        """
         # pylint: disable=R0914
         # Too many local variables - who cares?
+        savedir = filename = ""
 
-        def safe_filename(ff):
-            """ Remove troublesome characters in basename. """
-            d, b = os.path.split(ff)
-            ok = re.compile(r'[^\\/?*$\'"%&:<>|]')
-            b = "".join(x if ok.match(x) else "_" for x in b)
-            ff = os.path.join(d, b)
-            return ff.encode("utf8", "ignore")
+        if filepath and os.path.isdir(filepath):
+            savedir, filename = filepath, self.generate_filename()
+
+        elif filepath:
+            savedir, filename = os.path.split(filepath)
+
+        else:
+            filename = self.generate_filename(meta=meta)
+
+        filepath = os.path.join(savedir, filename)
+        temp_filepath = filepath + ".temp"
 
         status_string = ('  {:,} Bytes [{:.2%}] received. Rate: [{:4.0f} '
                          'KB/s].  ETA: [{:.0f} secs]')
@@ -707,35 +734,16 @@ class Stream(object):
         response = g.opener.open(self.url)
         total = int(response.info()['Content-Length'].strip())
         chunksize, bytesdone, t0 = 16384, 0, time.time()
-        fname = filepath or self.filename
-        dirname = os.path.dirname(fname)
 
-        if dirname and not os.path.isdir(dirname):
-            raise IOError("Invalid directory: %s" % dirname)
+        fmode, offset = "wb", 0
 
-        elif os.path.isdir(fname):
-            fname = os.path.join(fname, self.filename)
+        if os.path.exists(temp_filepath):
+            if os.stat(temp_filepath).st_size < total:
 
-        temp = os.path.join(os.path.dirname(fname), self._parent.title)
-        tempfname = "%s_%s_%s.part" % (temp, self._parent.videoid, self.itag)
-        safename, fmode, offset = safe_filename(tempfname), "wb", 0
+                offset = os.stat(temp_filepath).st_size
+                fmode = "ab"
 
-        if os.path.exists(tempfname) and os.stat(tempfname).st_size < total:
-            offset = os.stat(tempfname).st_size
-            fmode = "ab"
-
-        elif os.path.exists(safename) and os.stat(safename).st_size < total:
-            offset = os.stat(safename).st_size
-            fmode = "ab"
-
-        try:
-            outfh = open(tempfname, fmode)
-
-        except IOError:
-            # remove special chars from filename
-            fname = safe_filename(fname)
-            tempfname = safename
-            outfh = open(tempfname, fmode)
+        outfh = open(temp_filepath, fmode)
 
         if offset:
             # partial file exists, resume download
@@ -766,8 +774,8 @@ class Stream(object):
             if callback:
                 callback(total, *progress_stats)
 
-        os.rename(tempfname, fname)
-        return fname
+        os.rename(temp_filepath, filepath)
+        return filepath
 
 
 class Pafy(object):
@@ -777,14 +785,13 @@ class Pafy(object):
     funcmap = {}  # keep functions as a class variable
 
     def __init__(self, video_url, basic=True, gdata=False,
-                 signature=True, size=False, callback=None):
+                 signature=True, size=False, callback=lambda x: None):
         """ Set initial values. """
         self.version = __version__
         self.videoid = extract_video_id(video_url)
         self.watchv_url = g.urls['watchv'] % self.videoid
 
-        nullf = lambda x: None
-        new.callback = callback or nullf
+        new.callback = callback
         self._have_basic = False
         self._have_gdata = False
 
@@ -851,22 +858,24 @@ class Pafy(object):
             allinfo = get_video_info(self.videoid)
 
         new.callback("Fetched video info")
-        f = lambda x: allinfo.get(x, ["unknown"])[0]
-        t = lambda x: allinfo.get(x, [0.0])[0]
-        z = lambda x: allinfo.get(x, [""])[0]
 
-        self._title = f('title').replace("/", "-")
-        self._author = f('author')
-        self._videoid = f('video_id')
-        self._rating = float(t('avg_rating'))
-        self._length = int(f('length_seconds'))
-        self._viewcount = int(f('view_count'))
-        self._thumb = unquote_plus(f('thumbnail_url'))
-        self._formats = [x.split("/") for x in f('fmt_list').split(",")]
-        self._keywords = z('keywords').split(',')
-        self._bigthumb = z('iurlsd')
-        self._bigthumbhd = z('iurlsdmaxres')
-        self.ciphertag = f("use_cipher_signature") == "True"
+        def _get_lst(key, default="unknown", dic=allinfo):
+            """ Dict get function, returns first index. """
+            retval = dic.get(key, default)
+            return retval[0] if retval != default else default
+
+        self._title = _get_lst('title')
+        self._author = _get_lst('author')
+        self._videoid = _get_lst('video_id')
+        self._rating = float(_get_lst('avg_rating', 0.0))
+        self._length = int(_get_lst('length_seconds', 0))
+        self._viewcount = int(_get_lst('view_count'), 0)
+        self._thumb = unquote_plus(_get_lst('thumbnail_url', ""))
+        self._formats = [x.split("/") for x in _get_lst('fmt_list').split(",")]
+        self._keywords = _get_lst('keywords', "").split(',')
+        self._bigthumb = _get_lst('iurlsd', "")
+        self._bigthumbhd = _get_lst('iurlsdmaxres', "")
+        self.ciphertag = _get_lst("use_cipher_signature") == "True"
 
         if ageurl:
             self.ciphertag = False
@@ -1163,7 +1172,7 @@ class Pafy(object):
 
 
 def get_playlist(playlist_url, basic=False, gdata=False, signature=False,
-                 size=False, callback=None):
+                 size=False, callback=lambda x: None):
     """ Return a dict containing Pafy objects from a YouTube Playlist.
 
     The returned Pafy objects are initialised using the arguments to
@@ -1172,8 +1181,6 @@ def get_playlist(playlist_url, basic=False, gdata=False, signature=False,
     """
     # pylint: disable=R0914
     # too many local vars
-    nullf = lambda x: None
-    callback = callback or nullf
     x = (r"-_0-9a-zA-Z",) * 2 + (r'(?:\&|\#.{1,1000})',)
     regx = re.compile(r'(?:^|[^%s]+)([%s]{18,})(?:%s|$)' % x)
     m = regx.search(playlist_url)
